@@ -1,10 +1,11 @@
 /*****************************************************************************
- * RRDtool 1.0.49  Copyright Tobias Oetiker, 1997 - 2000
+ * RRDtool 1.2.11  Copyright by Tobi Oetiker, 1997-2005
  *****************************************************************************
  * pngsize.c  determine the size of a PNG image
  *****************************************************************************/
 
 #include <png.h>
+#include "rrd_tool.h"
 
 int
 PngSize(FILE *fd, long *width, long *height)
@@ -21,6 +22,13 @@ PngSize(FILE *fd, long *width, long *height)
   (*width)=0;
   (*height)=0;
 
+/* this is to make compile on aix work since they seem to define jmpbuf
+   to be _jmpbuf which breaks compilation */
+
+#ifdef jmpbuf
+#undef jmpbuf
+#endif
+
   if (setjmp(png_read_ptr->jmpbuf)){
     png_destroy_read_struct(&png_read_ptr, &info_ptr, (png_infopp)NULL);
     return 0;
