@@ -1,14 +1,25 @@
 /*****************************************************************************
- * RRDtool 1.0.50  Copyright Tobias Oetiker, 1999
+ * RRDtool 1.2.11  Copyright by Tobi Oetiker, 1997-2005
  * This code is stolen from rateup (mrtg-2.x) by Dave Rand
  *****************************************************************************
  * diff calculate the difference between two very long integers available as
  *      strings
  *****************************************************************************
- * $Id: rrd_diff.c,v 1.1.1.1 2002/02/26 10:21:37 oetiker Exp $
- * $Log: rrd_diff.c,v $
- * Revision 1.1.1.1  2002/02/26 10:21:37  oetiker
- * Intial Import
+ * $Id: rrd_diff.c 663 2005-07-25 14:12:58Z oetiker $
+ * $Log$
+ * Revision 1.4  2003/03/10 00:30:34  oetiker
+ * handle cases with two negative numbers
+ * --  Sasha Mikheev <sasha@avalon-net.co.il>
+ *
+ * Revision 1.3  2002/04/01 18:31:22  oetiker
+ * "!" takes a higher preference than "||" this means rrd_update N:: would
+ * segfault -- Oliver Cook <ollie@uk.clara.net>
+ *
+ * Revision 1.2  2002/02/01 20:34:49  oetiker
+ * fixed version number and date/time
+ *
+ * Revision 1.1.1.1  2001/02/25 22:25:05  oetiker
+ * checkin
  *
  * Revision 1.1  1998/10/08 18:21:45  oetiker
  * Initial revision
@@ -24,7 +35,8 @@
  *
  *****************************************************************************/
 
-#include "rrd_tool.h" 
+#include "rrd_tool.h"
+
 double
 rrd_diff(char *a, char *b)
 {
@@ -65,11 +77,13 @@ rrd_diff(char *a, char *b)
     r1[1] = 0;  /* Null terminate result */
     c = 0;
     for (x=0; x<m; x++) {
-        /* we want to avoid reading off the edge of the string */
-        char save_a,save_b;
-        save_a = ( a1 >= a) ? *a1 : '0';
-        save_b = ( b1 >= b) ? *b1 : '0';
-        *r1 = save_a - save_b - c + '0';
+        if (a1 >= a && b1 >= b) {
+            *r1 = ((*a1 - c) - *b1) + '0';
+        } else if (a1 >= a) {
+            *r1 = (*a1 - c);
+        } else {
+            *r1 = ('0' - *b1 - c) + '0';
+        }
         if (*r1 < '0') {
             *r1 += 10;
             c=1;
