@@ -1,5 +1,5 @@
 /*****************************************************************************
- * RRDtool 1.0.50  Copyright Tobias Oetiker, 1997 - 2000
+ * RRDtool 1.2.11  Copyright by Tobi Oetiker, 1997-2005
  *****************************************************************************
  * rrd_last.c
  *****************************************************************************
@@ -10,17 +10,25 @@
 
 time_t
 rrd_last(int argc, char **argv)
+{
+    if(argc < 2){
+        rrd_set_error("please specify an rrd");
+        return(-1);
+    }
+
+    return( rrd_last_r(argv[1]) );
+}
+ 
+
+time_t
+rrd_last_r(const char *filename)
 {
     FILE	*in_file;
     time_t       lastup;
 
     rrd_t	 rrd;
 
-    if(argc < 2){
-        rrd_set_error("please specify an rrd");
-        return(-1);
-    }
-    if(rrd_open(argv[1], &in_file, &rrd, RRD_READONLY)==-1){
+    if(rrd_open(filename, &in_file, &rrd, RRD_READONLY)==-1){
         return(-1);
     }
     lastup = rrd.live_head->last_up;
@@ -28,7 +36,5 @@ rrd_last(int argc, char **argv)
     fclose(in_file);
     return(lastup);
 }
- 
-
 
 
